@@ -21,41 +21,24 @@
 #ifndef __XFDESKTOP_FILE_ICON_MANAGER_H__
 #define __XFDESKTOP_FILE_ICON_MANAGER_H__
 
-#include <glib.h>
+#include <gtk/gtk.h>
 #include <xfconf/xfconf.h>
+#include <libxfce4windowing/libxfce4windowing.h>
 
+#include "xfdesktop-backdrop-manager.h"
 #include "xfdesktop-icon-view-manager.h"
 
 G_BEGIN_DECLS
 
-#define XFDESKTOP_TYPE_FILE_ICON_MANAGER     (xfdesktop_file_icon_manager_get_type())
-#define XFDESKTOP_FILE_ICON_MANAGER(obj)     (G_TYPE_CHECK_INSTANCE_CAST((obj), XFDESKTOP_TYPE_FILE_ICON_MANAGER, XfdesktopFileIconManager))
-#define XFDESKTOP_IS_FILE_ICON_MANAGER(obj)  (G_TYPE_CHECK_INSTANCE_TYPE((obj), XFDESKTOP_TYPE_FILE_ICON_MANAGER))
+G_DECLARE_FINAL_TYPE(XfdesktopFileIconManager, xfdesktop_file_icon_manager, XFDESKTOP, FILE_ICON_MANAGER, XfdesktopIconViewManager)
+#define XFDESKTOP_TYPE_FILE_ICON_MANAGER (xfdesktop_file_icon_manager_get_type())
 
-typedef struct _XfdesktopFileIconManager         XfdesktopFileIconManager;
-typedef struct _XfdesktopFileIconManagerClass    XfdesktopFileIconManagerClass;
-typedef struct _XfdesktopFileIconManagerPrivate  XfdesktopFileIconManagerPrivate;
-
-struct _XfdesktopFileIconManager
-{
-    XfdesktopIconViewManager parent;
-
-    /*< private >*/
-    XfdesktopFileIconManagerPrivate *priv;
-};
-
-struct _XfdesktopFileIconManagerClass
-{
-    XfdesktopIconViewManagerClass parent;
-
-    /*< signals >*/
-    void (*hidden_state_changed)(XfdesktopFileIconManager *fmanager);
-};
-
-GType xfdesktop_file_icon_manager_get_type(void) G_GNUC_CONST;
-
-XfdesktopIconViewManager *xfdesktop_file_icon_manager_new(XfconfChannel *channel,
-                                                          GtkWidget *parent,
+XfdesktopIconViewManager *xfdesktop_file_icon_manager_new(XfwScreen *screen,
+                                                          GdkScreen *gdkscreen,
+                                                          XfconfChannel *channel,
+                                                          GtkAccelGroup *accel_group,
+                                                          XfdesktopBackdropManager *backdrop_manager,
+                                                          GList *desktops,
                                                           GFile *folder);
 
 G_END_DECLS
