@@ -30,14 +30,17 @@
 #include "xfdesktop-file-utils.h"
 #include "xfdesktop-file-icon.h"
 
-struct _XfdesktopFileIconPrivate
+#define GET_PRIVATE(icon) ((XfdesktopFileIconPrivate *)xfdesktop_file_icon_get_instance_private(XFDESKTOP_FILE_ICON(icon)))
+
+typedef struct _XfdesktopFileIconPrivate
 {
     GIcon *gicon;
     gchar *sort_key;
-};
+} XfdesktopFileIconPrivate;
 
 static void xfdesktop_file_icon_finalize(GObject *obj);
 
+static gchar *xfdesktop_file_icon_get_identifier(XfdesktopIcon *icon);
 static gboolean xfdesktop_file_icon_activate(XfdesktopIcon *icon,
                                              GtkWindow *window);
 
@@ -69,29 +72,30 @@ xfdesktop_file_icon_class_init(XfdesktopFileIconClass *klass)
     gobject_class->set_property = xfdesktop_file_icon_set_property;
     gobject_class->get_property = xfdesktop_file_icon_get_property;
 
+    icon_class->get_identifier = xfdesktop_file_icon_get_identifier;
     icon_class->activate = xfdesktop_file_icon_activate;
 
     g_object_class_install_property(gobject_class,
                                     PROP_GICON,
-                                    g_param_spec_pointer("gicon",
-                                                         "gicon",
-                                                         "gicon",
-                                                         G_PARAM_READWRITE));
+                                    g_param_spec_object("gicon",
+                                                        "gicon",
+                                                        "gicon",
+                                                        G_TYPE_ICON,
+                                                        G_PARAM_READWRITE));
 }
 
 static void
-xfdesktop_file_icon_init(XfdesktopFileIcon *icon)
-{
-    icon->priv = xfdesktop_file_icon_get_instance_private(icon);
-}
+xfdesktop_file_icon_init(XfdesktopFileIcon *icon) {}
 
 static void
 xfdesktop_file_icon_finalize(GObject *obj)
 {
-    XfdesktopFileIcon *icon = XFDESKTOP_FILE_ICON(obj);
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(obj);
 
-    xfdesktop_file_icon_invalidate_icon(icon);
-    g_free(icon->priv->sort_key);
+    if (priv->gicon != NULL) {
+        g_object_unref(priv->gicon);
+    }
+    g_free(priv->sort_key);
 
     G_OBJECT_CLASS(xfdesktop_file_icon_parent_class)->finalize(obj);
 }
@@ -103,11 +107,12 @@ xfdesktop_file_icon_set_property(GObject *object,
                                  GParamSpec *pspec)
 {
     XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(object);
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(object);
 
     switch(property_id) {
         case PROP_GICON:
             xfdesktop_file_icon_invalidate_icon(file_icon);
-            file_icon->priv->gicon = g_value_get_pointer(value);
+            priv->gicon = g_value_dup_object(value);
             break;
 
         default:
@@ -122,11 +127,11 @@ xfdesktop_file_icon_get_property(GObject *object,
                                  GValue *value,
                                  GParamSpec *pspec)
 {
-    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(object);
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(object);
 
     switch(property_id) {
         case PROP_GICON:
-            g_value_set_pointer(value, file_icon->priv->gicon);
+            g_value_set_object(value, priv->gicon);
             break;
 
         default:
@@ -135,6 +140,21 @@ xfdesktop_file_icon_get_property(GObject *object,
     }
 }
 
+static gchar *
+xfdesktop_file_icon_get_identifier(XfdesktopIcon *icon) {
+    XfdesktopFileIcon *file_icon = XFDESKTOP_FILE_ICON(icon);
+
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
+
+    GFile *file = xfdesktop_file_icon_peek_file(file_icon);
+    gchar *identifier = g_file_get_path(file);
+    if (identifier == NULL) {
+        identifier = g_file_get_uri(file);
+    }
+
+    return identifier;
+}
+
 static gboolean
 xfdesktop_file_icon_activate(XfdesktopIcon *icon,
                              GtkWindow *window)
@@ -257,6 +277,18 @@ xfdesktop_file_icon_can_delete_file(XfdesktopFileIcon *icon)
         return FALSE;
 }
 
+gboolean
+xfdesktop_file_icon_is_hidden_file(XfdesktopFileIcon *icon) {
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), FALSE);
+
+    XfdesktopFileIconClass *klass = XFDESKTOP_FILE_ICON_GET_CLASS(icon);
+    if (klass->is_hidden_file != NULL) {
+        return klass->is_hidden_file(icon);
+    } else {
+        return FALSE;
+    }
+}
+
 GIcon *
 xfdesktop_file_icon_add_emblems(XfdesktopFileIcon *icon,
                                 GIcon *gicon)
@@ -304,18 +336,15 @@ xfdesktop_file_icon_invalidate_icon(XfdesktopFileIcon *icon)
 {
     g_return_if_fail(XFDESKTOP_IS_FILE_ICON(icon));
 
-    if(G_IS_ICON(icon->priv->gicon)) {
-        g_object_unref(icon->priv->gicon);
-        icon->priv->gicon = NULL;
-    }
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(icon);
+    g_clear_object(&priv->gicon);
 }
 
 gboolean
 xfdesktop_file_icon_has_gicon(XfdesktopFileIcon *icon)
 {
     g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), FALSE);
-
-    return G_IS_ICON(icon->priv->gicon);
+    return G_IS_ICON(GET_PRIVATE(icon)->gicon);
 }
 
 GIcon *
@@ -323,13 +352,14 @@ xfdesktop_file_icon_get_gicon(XfdesktopFileIcon *icon)
 {
     g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
 
-    if (icon->priv->gicon == NULL) {
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(icon);
+    if (priv->gicon == NULL) {
         XfdesktopFileIconClass *klass = XFDESKTOP_FILE_ICON_GET_CLASS(icon);
         g_return_val_if_fail(klass->get_gicon != NULL, NULL);
-        icon->priv->gicon = klass->get_gicon(icon);
+        priv->gicon = klass->get_gicon(icon);
     }
 
-    return icon->priv->gicon != NULL ? g_object_ref(icon->priv->gicon) : NULL;
+    return priv->gicon != NULL ? g_object_ref(priv->gicon) : NULL;
 }
 
 gdouble
@@ -356,8 +386,9 @@ xfdesktop_file_icon_peek_sort_key(XfdesktopFileIcon *icon)
 {
     g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), NULL);
 
-    if (icon->priv->sort_key != NULL) {
-        return icon->priv->sort_key;
+    XfdesktopFileIconPrivate *priv = GET_PRIVATE(icon);
+    if (priv->sort_key != NULL) {
+        return priv->sort_key;
     } else {
         XfdesktopFileIconClass *klass = XFDESKTOP_FILE_ICON_GET_CLASS(icon);
         gchar *sk = NULL;
@@ -372,18 +403,19 @@ xfdesktop_file_icon_peek_sort_key(XfdesktopFileIcon *icon)
         }
 
         if (G_LIKELY(sk != NULL)) {
-            icon->priv->sort_key = sk;
+            priv->sort_key = sk;
         }
         return sk;
     }
 }
 
 guint
-xfdesktop_file_icon_hash(gconstpointer icon)
+xfdesktop_file_icon_hash(gconstpointer data)
 {
     XfdesktopFileIconClass *klass;
 
-    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(icon), 0);
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON((gpointer)data), 0);
+    XfdesktopFileIcon *icon = XFDESKTOP_FILE_ICON((gpointer)data);
 
     klass = XFDESKTOP_FILE_ICON_GET_CLASS(icon);
     if (klass->hash != NULL) {
@@ -403,9 +435,9 @@ gint
 xfdesktop_file_icon_equal(gconstpointer a,
                           gconstpointer b)
 {
-    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(a), 0);
-    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON(b), 0);
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON((gpointer)a), 0);
+    g_return_val_if_fail(XFDESKTOP_IS_FILE_ICON((gpointer)b), 0);
 
-    return g_str_equal(xfdesktop_file_icon_peek_sort_key(XFDESKTOP_FILE_ICON(a)),
-                       xfdesktop_file_icon_peek_sort_key(XFDESKTOP_FILE_ICON(b)));
+    return g_str_equal(xfdesktop_file_icon_peek_sort_key(XFDESKTOP_FILE_ICON((gpointer)a)),
+                       xfdesktop_file_icon_peek_sort_key(XFDESKTOP_FILE_ICON((gpointer)b)));
 }
