@@ -123,15 +123,11 @@ xfdesktop_x11_desktop_scrolled(GtkWidget *widget, GdkEventScroll *event) {
 }
 
 void
-xfdesktop_x11_set_root_image_file_property(GdkScreen *gscreen, guint monitor_num, const gchar *filename) {
-    GdkDisplay *display;
-    gchar *property_name;
-
-    display = gdk_screen_get_display(gscreen);
+xfdesktop_x11_set_root_image_file_property(GdkScreen *gscreen, gint monitor_idx, const gchar *filename) {
+    GdkDisplay *display = gdk_screen_get_display(gscreen);
+    gchar *property_name = g_strdup_printf(XFDESKTOP_IMAGE_FILE_FMT, monitor_idx);
 
     gdk_x11_display_error_trap_push(display);
-
-    property_name = g_strdup_printf(XFDESKTOP_IMAGE_FILE_FMT, monitor_num);
     if (filename != NULL) {
         gdk_property_change(gdk_screen_get_root_window(gscreen),
                             gdk_atom_intern(property_name, FALSE),
@@ -144,7 +140,6 @@ xfdesktop_x11_set_root_image_file_property(GdkScreen *gscreen, guint monitor_num
         gdk_property_delete(gdk_screen_get_root_window(gscreen),
                             gdk_atom_intern(property_name, FALSE));
     }
-
     gdk_x11_display_error_trap_pop_ignored(display);
 
     g_free(property_name);
@@ -153,34 +148,40 @@ xfdesktop_x11_set_root_image_file_property(GdkScreen *gscreen, guint monitor_num
 void
 xfdesktop_x11_set_root_image_surface(GdkScreen *gscreen, cairo_surface_t *surface) {
 #ifndef DISABLE_FOR_BUG7442
-    Pixmap pixmap_id;
-    GdkDisplay *display;
-    GdkWindow *groot;
-    cairo_pattern_t *pattern;
+    GdkWindow *groot = gdk_screen_get_root_window(gscreen);
+    GdkAtom prop_atom = gdk_atom_intern("_XROOTPMAP_ID", FALSE);
+    cairo_pattern_t *pattern = NULL;
 
-    groot = gdk_screen_get_root_window(gscreen);
-    pixmap_id = cairo_xlib_surface_get_drawable (surface);
+    if (surface != NULL) {
+        Pixmap pixmap_id = cairo_xlib_surface_get_drawable(surface);
+        pattern = cairo_pattern_create_for_surface(surface);
 
-    display = gdk_screen_get_display(gscreen);
-    xfw_windowing_error_trap_push(display);
+        GdkDisplay *display = gdk_screen_get_display(gscreen);
+        xfw_windowing_error_trap_push(display);
+
+        /* set root property for transparent Eterms */
+        gdk_property_change(groot,
+                            prop_atom,
+                            gdk_atom_intern("PIXMAP", FALSE),
+                            32,
+                            GDK_PROP_MODE_REPLACE,
+                            (guchar *)&pixmap_id,
+                            1);
+        /* there really should be a standard for this crap... */
+
+        xfw_windowing_error_trap_pop_ignored(display);
+    } else {
+        gdk_property_delete(groot, prop_atom);
+    }
 
-    /* set root property for transparent Eterms */
-    gdk_property_change(groot,
-                        gdk_atom_intern("_XROOTPMAP_ID", FALSE),
-                        gdk_atom_intern("PIXMAP", FALSE),
-                        32,
-                        GDK_PROP_MODE_REPLACE,
-                        (guchar *)&pixmap_id,
-                        1);
-    /* and set the root window's BG surface, because aterm is somewhat lame. */
-    pattern = cairo_pattern_create_for_surface(surface);
 G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+    /* and set the root window's BG surface, because aterm is somewhat lame. */
     gdk_window_set_background_pattern(groot, pattern);
 G_GNUC_END_IGNORE_DEPRECATIONS
-    cairo_pattern_destroy(pattern);
-    /* there really should be a standard for this crap... */
 
-    xfw_windowing_error_trap_pop_ignored(display);
+    if (pattern != NULL) {
+        cairo_pattern_destroy(pattern);
+    }
 #endif
 }
 
@@ -229,11 +230,11 @@ xfdesktop_x11_set_desktop_manager_selection(GdkScreen *gscreen, GError **error)
 
     gchar *common_selection_name = g_strdup_printf("_NET_DESKTOP_MANAGER_S%d", screen_num);
     GdkAtom common_selection_atom = gdk_atom_intern(common_selection_name, FALSE);
-    g_free(common_selection_name);
 
     gchar *xfce_selection_name = g_strdup_printf(XFDESKTOP_SELECTION_FMT, screen_num);
     GdkAtom xfce_selection_atom = gdk_atom_intern(xfce_selection_name, FALSE);
-    g_free(xfce_selection_name);
+
+    GdkWindow *selection_window = NULL;
 
     // We have to use Xlib for these, as the GDK functions only return
     // the selection owner if it's owned by a GdkWindow known to GDK,
@@ -243,89 +244,85 @@ xfdesktop_x11_set_desktop_manager_selection(GdkScreen *gscreen, GError **error)
                              G_IO_ERROR_ADDRESS_IN_USE,
                              "Another desktop manager is already running on screen %d",
                              screen_num);
-        return NULL;
-    }
-    if (XGetSelectionOwner(display, gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom)) != None) {
+    } else if (XGetSelectionOwner(display, gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom)) != None) {
         *error = g_error_new(G_IO_ERROR,
                              G_IO_ERROR_ADDRESS_IN_USE,
                              "Another instance of xfdesktop is already running on screen %d",
                              screen_num);
-        return NULL;
+    } else {
+        GdkWindowAttr attrs = {
+            .window_type = GDK_WINDOW_TOPLEVEL,
+            .override_redirect = TRUE,
+            .width = 1,
+            .height = 1,
+            .x = -100,
+            .y = -100,
+            .wmclass_name = "xfdesktop",
+            .wmclass_class = "Xfdesktop",
+            .title = "Xfdesktop Manager Selection",
+            .event_mask = GDK_BUTTON_PRESS |
+                GDK_PROPERTY_CHANGE_MASK |
+                GDK_STRUCTURE_MASK |
+                GDK_SUBSTRUCTURE_MASK,
+        };
+        selection_window = gdk_window_new(gdk_screen_get_root_window(gscreen),
+                                          &attrs,
+                                          GDK_WA_TITLE |
+                                          GDK_WA_WMCLASS |
+                                          GDK_WA_X |
+                                          GDK_WA_Y |
+                                          GDK_WA_NOREDIR);
+
+        if (!gdk_selection_owner_set_for_display(gdisplay,
+                                                 selection_window,
+                                                 common_selection_atom,
+                                                 GDK_CURRENT_TIME,
+                                                 TRUE))
+        {
+            *error = g_error_new(G_IO_ERROR,
+                                 G_IO_ERROR_FAILED,
+                                 "Unable to acquire selection '%s'",
+                                 common_selection_name);
+        } else if (!gdk_selection_owner_set_for_display(gdisplay,
+                                                        selection_window,
+                                                        xfce_selection_atom,
+                                                        GDK_CURRENT_TIME,
+                                                        TRUE))
+        {
+            *error = g_error_new(G_IO_ERROR,
+                                 G_IO_ERROR_FAILED,
+                                 "Unable to acquire selection '%s'",
+                                 xfce_selection_name);
+        } else {
+            Window new_owner = XGetSelectionOwner(display, gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom));
+            if (new_owner != gdk_x11_window_get_xid(selection_window)) {
+                *error = g_error_new(G_IO_ERROR,
+                                     G_IO_ERROR_FAILED,
+                                     "Failed to acquire selection '%s'",
+                                     xfce_selection_name);
+            } else {
+                Window xroot = gdk_x11_window_get_xid(gdk_screen_get_root_window(gscreen));
+                XClientMessageEvent xev;
+                xev.type = ClientMessage;
+                xev.window = xroot;
+                xev.message_type = XInternAtom(display, "MANAGER", False);
+                xev.format = 32;
+                xev.data.l[0] = GDK_CURRENT_TIME;
+                xev.data.l[1] = gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom);
+                xev.data.l[2] = gdk_x11_window_get_xid(selection_window);
+                xev.data.l[3] = 0;    /* manager specific data */
+                xev.data.l[4] = 0;    /* manager specific data */
+                XSendEvent(display, xroot, False, StructureNotifyMask, (XEvent *)&xev);
+            }
+        }
     }
 
-    GdkWindowAttr attrs = {
-        .window_type = GDK_WINDOW_TOPLEVEL,
-        .override_redirect = TRUE,
-        .width = 1,
-        .height = 1,
-        .x = -100,
-        .y = -100,
-        .wmclass_name = "xfdesktop",
-        .wmclass_class = "Xfdesktop",
-        .title = "Xfdesktop Manager Selection",
-        .event_mask = GDK_BUTTON_PRESS |
-            GDK_PROPERTY_CHANGE_MASK |
-            GDK_STRUCTURE_MASK |
-            GDK_SUBSTRUCTURE_MASK,
-    };
-    GdkWindow *selection_window = gdk_window_new(gdk_screen_get_root_window(gscreen),
-                                                 &attrs,
-                                                 GDK_WA_TITLE |
-                                                 GDK_WA_WMCLASS |
-                                                 GDK_WA_X |
-                                                 GDK_WA_Y |
-                                                 GDK_WA_NOREDIR);
-
-    if (!gdk_selection_owner_set_for_display(gdisplay,
-                                             selection_window,
-                                             common_selection_atom,
-                                             GDK_CURRENT_TIME,
-                                             TRUE))
-    {
-        *error = g_error_new(G_IO_ERROR,
-                             G_IO_ERROR_FAILED,
-                             "Unable to acquire selection '%s'",
-                             gdk_atom_name(common_selection_atom));
-        gdk_window_destroy(selection_window);
-        return NULL;
+    if (*error != NULL) {
+        g_clear_pointer(&selection_window, gdk_window_destroy);
     }
 
-    if (!gdk_selection_owner_set_for_display(gdisplay,
-                                             selection_window,
-                                             xfce_selection_atom,
-                                             GDK_CURRENT_TIME,
-                                             TRUE))
-    {
-        *error = g_error_new(G_IO_ERROR,
-                             G_IO_ERROR_FAILED,
-                             "Unable to acquire selection '%s'",
-                             gdk_atom_name(xfce_selection_atom));
-        gdk_window_destroy(selection_window);
-        return NULL;
-    }
-
-    Window new_owner = XGetSelectionOwner(display, gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom));
-    if (new_owner != gdk_x11_window_get_xid(selection_window)) {
-        gdk_window_destroy(selection_window);
-        *error = g_error_new(G_IO_ERROR,
-                             G_IO_ERROR_FAILED,
-                             "Failed to acquire selection '%s'",
-                             gdk_atom_name(xfce_selection_atom));
-        return NULL;
-    }
-
-    Window xroot = gdk_x11_window_get_xid(gdk_screen_get_root_window(gscreen));
-    XClientMessageEvent xev;
-    xev.type = ClientMessage;
-    xev.window = xroot;
-    xev.message_type = XInternAtom(display, "MANAGER", False);
-    xev.format = 32;
-    xev.data.l[0] = GDK_CURRENT_TIME;
-    xev.data.l[1] = gdk_x11_atom_to_xatom_for_display(gdisplay, xfce_selection_atom);
-    xev.data.l[2] = gdk_x11_window_get_xid(selection_window);
-    xev.data.l[3] = 0;    /* manager specific data */
-    xev.data.l[4] = 0;    /* manager specific data */
-    XSendEvent(display, xroot, False, StructureNotifyMask, (XEvent *)&xev);
+    g_free(common_selection_name);
+    g_free(xfce_selection_name);
 
     return selection_window;
 }
